#ifndef _Conn_com_h
#define _Conn_com_h 1

#define _GNU_SOURCE

#include <sys/types.h>
#include <stdio.h>
#include <time.h>


/* type */
#define	Conn_type_MASTER	1
#define Conn_type_CLIENT	2
#define Conn_type_UNK		3


/* state */
#define CONN_STATE_START	0
#define CONN_STATE_FREE		CONN_STATE_START + 1
#define CONN_STATE_EMPTY	CONN_STATE_START + 2
#define CONN_STATE_OPEN		CONN_STATE_START + 3
#define CONN_STATE_LISTEN	CONN_STATE_START + 4
#define CONN_STATE_CONNECT_0	CONN_STATE_START + 5
#define CONN_STATE_CONNECT_a	CONN_STATE_START + 6
#define CONN_STATE_CONNECT_b	CONN_STATE_START + 7


/* error kind */
#define CONN_ERROR_START	0
#define CONN_ERROR_USERREQ	CONN_ERROR_START +  1		/* user requested the close */
#define CONN_ERROR_POLL		CONN_ERROR_START +  2
#define CONN_ERROR_RECV		CONN_ERROR_START +  3
#define CONN_ERROR_SEND		CONN_ERROR_START +  4
#define CONN_ERROR_SOCKET	CONN_ERROR_START +  5
#define CONN_ERROR_HANGUP	CONN_ERROR_START +  6
#define CONN_ERROR_GETADDRINFO	CONN_ERROR_START +  7
#define CONN_ERROR_EXPIRED	CONN_ERROR_START +  8
#define CONN_ERROR_ACCEPT	CONN_ERROR_START +  9
#define CONN_ERROR_MEM		CONN_ERROR_START + 10
#define CONN_ERROR_CONNECT	CONN_ERROR_START + 11


/* FLAGS */
#define CONN_FLAGS_AUTO_RECONNECT	0x01 << 0


/* Parameters */
#define CONN_PARA_START			0
#define CONN_PARA_AUTO_RECONNECT	CONN_PARA_START + 1
#define CONN_PARA_RECONNECT_DELAY	CONN_PARA_START + 2
#define CONN_IDLE_TIME			CONN_PARA_START + 3



typedef struct
{
	unsigned char		type;

	unsigned char		state;
	unsigned char		error_state;

	unsigned int		slot;

	char			*ibuf;
	unsigned int		ibuf_size, ibuf_head, ibuf_tail;

	char			*obuf;
	unsigned int		obuf_size, obuf_head, obuf_tail;

	time_t			trecv;		/* last time we saw an receive */
	unsigned int		idle;		/* idle time allowed */

	int			sock_domain;
	int			sock_type;

	char			addr[64];
	int			port, via;	/* "via" is the port via a client was accepted */

	unsigned long long	bi, bo;

	void			*private;	/* private use by user */

	/* reconnect stuff */
	unsigned int		retries, timeout;
	unsigned int		delay; /* delay between reconnects */
	unsigned int		tryat; /* when we go to CONNECT_a state */

	int			xerrno;

	time_t			start;

	unsigned long		flags;

	/* bandwidth stuff */
	struct timeval		band_lasttime;	/* last time tokens were added */
	unsigned int		band_tokens;	/* 1 token -> 1000 bytes */
	unsigned int		band_factor;	/* tokens cannot go past f * w */
	unsigned int		band_width;	/* in 1000b increments */

	unsigned long long	id;		/* the id of a connection */
} Conn;



#endif
